==> threelet-0.9.13.tgz: package/README.md <==
threelet
========

**threelet** is a [three.js](https://github.com/mrdoob/three.js/) based
scene viewer. It can be also used as a 3D app framework for rapidly
making dynamic desktop/VR applications. Some notable features include:

-   3D scene update interface,
-   built-in render loop manager, and
-   mouse/vr-controller event handlers.

**demos**

-   Hello world (with the default axes and a unit lattice) \[
    [live](https://w3reality.github.io/threelet/examples/simple/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/hello-world-with-threelet)
    \]
-   Hello VR world \[
    [live](https://w3reality.github.io/threelet/examples/simple-webvr/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple-webvr/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/hello-world-with-threelet/2)
    \]
-   App with a static scene (mouse-event driven passive rendering) \[
    [live](https://w3reality.github.io/threelet/examples/simple-static/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple-static/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/making-a-static-3d-app)
    \]
-   App with a dynamic scene (rendering at 30 fps) \[
    [live](https://w3reality.github.io/threelet/examples/simple-dynamic/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple-dynamic/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/making-a-dynamic-3d-app)
    \]
-   WebVR controller state visualizer \[
    [live](https://w3reality.github.io/threelet/examples/webvr-controllers/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/webvr-controllers)
    \]
-   VR app with interactive objects \[
    [live](https://w3reality.github.io/threelet/examples/webvr-interactive/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/webvr-interactive/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/making-an-interactive-vr-app)
    \]
-   Embedding a 3D viewer into a web page. \[
    [live](https://w3reality.github.io/threelet/examples/embed-inline-block/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/embed-inline-block/index.html)
    \]
-   Embedding multiple independent viewers into a web page. \[
    [live](https://w3reality.github.io/threelet/examples/embed-multiple/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/embed-multiple/index.html)
    \]
-   Hello glTF animation. \[
    [live](https://w3reality.github.io/threelet/examples/animation-hello/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/animation-hello/index.html)
    \]
-   Animation player (with glTF, FBX and Collada models). \[
    [live](https://w3reality.github.io/threelet/examples/animation-player/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/animation-player/index.html)
    \]
-   In-window VR casting. \[
    [live](https://w3reality.github.io/threelet/examples/vr-casting-in-window/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/vr-casting-in-window/index.html)
    \]

[![image](https://w3reality.github.io/threelet/examples/simple-dynamic/index.jpg)](https://w3reality.github.io/threelet/examples/simple-dynamic/index.html)

[![image](https://w3reality.github.io/threelet/examples/media/img/mora2.jpg)](https://w3reality.github.io/threelet/examples/animation-player/index.html)

[![image](https://w3reality.github.io/threelet/examples/media/img/casting-2.jpg)](https://w3reality.github.io/threelet/examples/vr-casting-in-window/index.html)

Setup
-----

**Installation**

``` {.sourceCode .sh}
$ npm install threelet
```

**Loading**

ES6:

``` {.sourceCode .js}
import Threelet from 'src/index.js';
```

Script tag:

``` {.sourceCode .html}
<script src="dist/threelet.min.js"></script>
```

Hello world
-----------

``` {.sourceCode .html}
<canvas id="canvas" style="width: 100%; height: 100%;"></canvas>

<script src="../deps/three.min.js"></script>
<script src="../deps/OrbitControls.js"></script>
<script src="../deps/stats.min.js"></script>

<script src="../../dist/threelet.min.js"></script>

<script>
const threelet = new Threelet({
    canvas: document.getElementById('canvas'),
});

threelet.setup('mod-controls', THREE.OrbitControls);
threelet.setup('mod-stats', window.Stats);

threelet.render(); // first time
</script>
```

[live](https://w3reality.github.io/threelet/examples/simple/index.html)
\| [source
code](https://github.com/w3reality/threelet/tree/master/examples/simple/index.html)

![image](https://w3reality.github.io/threelet/examples/simple/img/threelet.png)

More usage
----------

**Basic**

`camera`, `scene` and `renderer` can be automatically/manually
configured:

``` {.sourceCode .js}
const threelet = new Threelet({canvas: myCanvas});
// now the following objects are all set
//   threelet.camera
//   threelet.scene (with the default axes and a unit lattice)
//   threelet.renderer
```

`scene` can be customized as:

``` {.sourceCode .js}
const threelet = new Threelet({
    canvas: myCanvas,
    scene: myScene, // instantiate with a custom scene
});

threelet.scene.add(myObject) // add an object to the scene
```

render modes (passive, active, and fps-throttled) by the built-in loop
controller:

``` {.sourceCode .js}
threelet.updateLoop(fps); // render at fps using the looper

threelet.render(); // atomic render manually
```

scene mechanics handler:

``` {.sourceCode .js}
threelet.update = (t, dt) => {
   // your implementation
};
```

`dispose()` terminates the loop and disposes all the scene objects:

``` {.sourceCode .js}
threelet.dispose();
```

**Parameters**

Calling the constructor with the default parameters looks as:

``` {.sourceCode .js}
const threelet = new Threelet({
    canvas: null,
    width: 480,
    height: 320,
    // ---- viewer options ----
    optScene: null,
    optAxes: true, // axes and a unit lattice
    optCameraPosition: [0, 1, 2], // initial camera position in desktop mode
});
```

**Embedding**

Without the `canvas` parameter, the constructor creates an inline-block
div element (`threelet.domElement`) that is ready to be embedded into a
web page.

``` {.sourceCode .html}
<div>
    This <span id="viewer"></span> is an inline-block element.
</div>

const threelet = new Threelet({width: 480, height: 320});
document.getElementById('viewer').appendChild(threelet.domElement);
```

**Event listeners**

``` {.sourceCode .js}
// setting mouse listeners
//   mx, my: mouse coordinates

threelet.on('mouse-click', (mx, my) => { /* ... */ }); // alias of 'mouse-click-left'
threelet.on('mouse-click-left', (mx, my) => { /* ... */ });
threelet.on('mouse-click-middle', (mx, my) => { /* ... */ });
threelet.on('mouse-click-right', (mx, my) => { /* ... */ });

threelet.on('mouse-down', (mx, my) => { /* ... */ });  // alias of 'mouse-down-left'
threelet.on('mouse-down-left', (mx, my) => { /* ... */ });
threelet.on('mouse-down-middle', (mx, my) => { /* ... */ });
threelet.on('mouse-down-right', (mx, my) => { /* ... */ });

threelet.on('mouse-move', (mx, my) => { /* ... */ });
threelet.on('mouse-drag-end', (mx, my) => { /* ... */ });

// setting VR controller listeners
//   i: controller index
//   x, y: touchpad coordinates

threelet.on('vr-touchpad-touch-start', (i, x, y) => { /* ... */ });
threelet.on('vr-touchpad-touch-end', (i, x, y) => { /* ... */ });
threelet.on('vr-touchpad-press-start', (i, x, y) => { /* ... */ });
threelet.on('vr-touchpad-press-end', (i, x, y) => { /* ... */ });

threelet.on('vr-trigger-press-start', (i) => { /* ... */ });
threelet.on('vr-trigger-press-end', (i) => { /* ... */ });

// cancelling

threelet.on(eventName, null);
```

**Utils**

raycasting:

``` {.sourceCode .js}
threelet.raycast(origin, direction, meshes, recursive=false, faceExclude=null);
threelet.raycastFromMouse(mx, my, meshes, recursive=false); // mx, my: mouse coordinates
threelet.raycastFromController(i, meshes, recursive=false); // i: VR controller index
```

**Plugin features**

OrbitControls, stats, and WebVR:

``` {.sourceCode .html}
<script src="OrbitControls.js"></script>
<script src="stats.min.js"></script>
<script src="WebVR.js"></script>
```

``` {.sourceCode .js}
threelet.setup('mod-controls', THREE.OrbitControls); // enable controls
threelet.setup('mod-stats', window.Stats); // show the stats meter
threelet.setup('mod-webvr', window.WEBVR); // show the desktop/VR switch button
```

Sky based on the
[shaders/sky](https://threejs.org/examples/?q=sky#webgl_shaders_sky)
example in three.js:

``` {.sourceCode .html}
<script src="Sky.js"></script>

threelet.setup('mod-sky', THREE.Sky); // show sky with the analytical daylight
```

Build
-----

``` {.sourceCode .}
$ npm install  # set up build tools
$ npm run build  # generate module files in lib/
```

==> threelet-0.9.14.tgz: package/README.md <==
threelet
========

**threelet** is a [three.js](https://github.com/mrdoob/three.js/) based
scene viewer. It can be also used as a 3D app framework for rapidly
making dynamic desktop/VR applications. Some notable features include:

-   built-in render loop manager (with auto VR context switching),
-   function interface `.update = (t, dt) => {}` for programming
    temporal 3D scenes, and
-   easy-to-use mouse/vr-controller event listeners.

**Basic demos**

-   Hello world (with the default axes and a unit lattice) \[
    [live](https://w3reality.github.io/threelet/examples/simple/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/hello-world-with-threelet)
    \]
-   Hello VR world \[
    [live](https://w3reality.github.io/threelet/examples/simple-webvr/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple-webvr/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/hello-world-with-threelet/2)
    \]
-   App with a static scene (mouse-event driven passive rendering) \[
    [live](https://w3reality.github.io/threelet/examples/simple-static/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple-static/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/making-a-static-3d-app)
    \]
-   App with a dynamic scene (rendering at 30 fps) \[
    [live](https://w3reality.github.io/threelet/examples/simple-dynamic/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/simple-dynamic/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/making-a-dynamic-3d-app)
    \]
-   Embedding a 3D viewer into a web page. \[
    [live](https://w3reality.github.io/threelet/examples/embed-inline-block/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/embed-inline-block/index.html)
    \]
-   Embedding multiple independent viewers into a web page. \[
    [live](https://w3reality.github.io/threelet/examples/embed-multiple/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/embed-multiple/index.html)
    \]
-   Hello glTF animation. \[
    [live](https://w3reality.github.io/threelet/examples/animation-hello/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/animation-hello/index.html)
    \]

**App demos**

-   VR app with interactive objects \[
    [live](https://w3reality.github.io/threelet/examples/webvr-interactive/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/webvr-interactive/index.html)
    \|
    [Observable](https://observablehq.com/@j-devel/making-an-interactive-vr-app)
    \]
-   WebVR controller state visualizer \[
    [live](https://w3reality.github.io/threelet/examples/webvr-controllers/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/webvr-controllers)
    \]
-   Animation player (with glTF, FBX and Collada models). \[
    [live](https://w3reality.github.io/threelet/examples/animation-player/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/animation-player/index.html)
    \]
-   glTF model selection panel. \[
    [live](https://w3reality.github.io/threelet/examples/model-selection/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/model-selection/index.html)
    \]
-   In-window VR casting. \[
    [live](https://w3reality.github.io/threelet/examples/vr-casting-in-window/index.html)
    \|
    [source](https://github.com/w3reality/threelet/tree/master/examples/vr-casting-in-window/index.html)
    \]

**Screenshots**

-   

    [App with a dynamic scene](https://w3reality.github.io/threelet/examples/simple-dynamic/index.html)

    :   [![image](https://w3reality.github.io/threelet/examples/simple-dynamic/index.jpg){width="800px"}](https://w3reality.github.io/threelet/examples/simple-dynamic/index.html)

-   

    [Animation player](https://w3reality.github.io/threelet/examples/animation-player/index.html)

    :   [![image](https://w3reality.github.io/threelet/examples/media/img/mora2.jpg)](https://w3reality.github.io/threelet/examples/animation-player/index.html)

-   

    [In-window VR casting](https://w3reality.github.io/threelet/examples/vr-casting-in-window/index.html)

    :   [![image](https://w3reality.github.io/threelet/examples/media/img/casting-2.jpg)](https://w3reality.github.io/threelet/examples/vr-casting-in-window/index.html)

Setup
-----

**Installation**

``` {.sourceCode .sh}
$ npm install threelet
```

**Loading**

ES6:

``` {.sourceCode .js}
import Threelet from 'src/index.js';
```

Script tag:

``` {.sourceCode .html}
<script src="dist/threelet.min.js"></script>
```

Hello world
-----------

``` {.sourceCode .html}
<canvas id="canvas" style="width: 100%; height: 100%;"></canvas>

<script src="../deps/three.min.js"></script>
<script src="../deps/OrbitControls.js"></script>
<script src="../deps/stats.min.js"></script>

<script src="../../dist/threelet.min.js"></script>

<script>
const threelet = new Threelet({
    canvas: document.getElementById('canvas'),
});

threelet.setup('mod-controls', THREE.OrbitControls);
threelet.setup('mod-stats', window.Stats);

threelet.render(); // first time
</script>
```

[live](https://w3reality.github.io/threelet/examples/simple/index.html)
\| [source
code](https://github.com/w3reality/threelet/tree/master/examples/simple/index.html)

[![image](https://w3reality.github.io/threelet/examples/simple/img/threelet.png)](https://w3reality.github.io/threelet/examples/simple/index.html)

More usage
----------

**Basic**

`camera`, `scene` and `renderer` can be automatically/manually
configured:

``` {.sourceCode .js}
const threelet = new Threelet({canvas: myCanvas});
// now the following objects are all set
//   threelet.camera
//   threelet.scene (with the default axes and a unit lattice)
//   threelet.renderer
```

`scene` can be customized as:

``` {.sourceCode .js}
const threelet = new Threelet({
    canvas: myCanvas,
    scene: myScene, // instantiate with a custom scene
});

threelet.scene.add(myObject) // add an object to the scene
```

specifying render modes (passive, active, and fps-throttled) by the
built-in loop controller:

``` {.sourceCode .js}
threelet.updateLoop(fps); // render at fps using the looper

threelet.render(); // atomic render manually
```

programming 3D scene dynamics
([example](https://w3reality.github.io/threelet/examples/simple-dynamic/index.html)
\|
[source](https://github.com/w3reality/threelet/tree/master/examples/simple-dynamic/index.html)):

``` {.sourceCode .js}
threelet.update = (t, dt) => {
   // your implementation
};
```

`dispose()` terminates the loop and disposes all the scene objects:

``` {.sourceCode .js}
threelet.dispose();
```

**Parameters**

Calling the constructor with the default parameters looks as:

``` {.sourceCode .js}
const threelet = new Threelet({
    canvas: null,
    width: 480,
    height: 320,
    // ---- viewer options ----
    optScene: null,
    optAxes: true, // axes and a unit lattice
    optCameraPosition: [0, 1, 2], // initial camera position in desktop mode
});
```

**Embedding**

Without the `canvas` parameter, the constructor creates an inline-block
div element (`threelet.domElement`) that is ready to be embedded into a
web page. ([single
case](https://w3reality.github.io/threelet/examples/embed-inline-block/index.html)
\| [multiple
case](https://w3reality.github.io/threelet/examples/embed-multiple/index.html))

``` {.sourceCode .html}
<div>
    This <span id="viewer"></span> is an inline-block element.
</div>

<script>
const threelet = new Threelet({width: 480, height: 320});
document.getElementById('viewer').appendChild(threelet.domElement);
</script>
```

**Event listeners**

setting mouse listeners
([example](https://w3reality.github.io/threelet/examples/model-selection/index.html)
\|
[source](https://github.com/w3reality/threelet/tree/master/examples/model-selection/index.html)):

``` {.sourceCode .js}
// mx, my: mouse coordinates

threelet.on('mouse-click', (mx, my) => { /* ... */ }); // alias of 'mouse-click-left'
threelet.on('mouse-click-left', (mx, my) => { /* ... */ });
threelet.on('mouse-click-middle', (mx, my) => { /* ... */ });
threelet.on('mouse-click-right', (mx, my) => { /* ... */ });

threelet.on('mouse-down', (mx, my) => { /* ... */ });  // alias of 'mouse-down-left'
threelet.on('mouse-down-left', (mx, my) => { /* ... */ });
threelet.on('mouse-down-middle', (mx, my) => { /* ... */ });
threelet.on('mouse-down-right', (mx, my) => { /* ... */ });

threelet.on('mouse-move', (mx, my) => { /* ... */ });
threelet.on('mouse-drag-end', (mx, my) => { /* ... */ });
```

setting VR controller listeners
([example](https://w3reality.github.io/threelet/examples/webvr-controllers/index.html)
\|
[source](https://github.com/w3reality/threelet/tree/master/examples/webvr-controllers/index.html)):

``` {.sourceCode .js}
// i: controller index
// x, y: touchpad coordinates

threelet.on('vr-touchpad-touch-start', (i, x, y) => { /* ... */ });
threelet.on('vr-touchpad-touch-end', (i, x, y) => { /* ... */ });
threelet.on('vr-touchpad-press-start', (i, x, y) => { /* ... */ });
threelet.on('vr-touchpad-press-end', (i, x, y) => { /* ... */ });

threelet.on('vr-trigger-press-start', (i) => { /* ... */ });
threelet.on('vr-trigger-press-end', (i) => { /* ... */ });
```

unsetting listeners:

``` {.sourceCode .js}
threelet.on(eventName, null);
```

**Raycasting**

``` {.sourceCode .js}
threelet.raycast(origin, direction, meshes, recursive=false, faceExclude=null);
threelet.raycastFromMouse(mx, my, meshes, recursive=false); // mx, my: mouse coordinates
threelet.raycastFromController(i, meshes, recursive=false); // i: VR controller index
```

**Utils**

animation loading
([example](https://w3reality.github.io/threelet/examples/animation-hello/index.html)
\|
[source](https://github.com/w3reality/threelet/tree/master/examples/animation-hello/index.html)):

``` {.sourceCode .js}
// <script src="../deps/GLTFLoader.js"></script>
const data = await Threelet.Utils.loadGLTF(path, file);

// <script src="../deps/inflate.min.js"></script>
// <script src="../deps/FBXLoader.js"></script>
const data = await Threelet.Utils.loadFBX(path);

// <script src="../deps/ColladaLoader.js"></script>
const data = await Threelet.Utils.loadCollada(path);
```

creating test THREE objects (used in the examples for shortcuts):

``` {.sourceCode .js}
const obj = Threelet.Utils.createTestHemisphereLight();
const obj = Threelet.Utils.createTestDirectionalLight();
const obj = Threelet.Utils.createTestCube(size=[0.4, 0.1, 0.4], color=0xff00ff, wireframe=false);
const objs = Threelet.Utils.createTestObjects(offset=[0, 1, -2]);
```

**Plugin features**

OrbitControls, stats, and WebVR:

``` {.sourceCode .html}
<script src="OrbitControls.js"></script>
<script src="stats.min.js"></script>
<script src="WebVR.js"></script>
```

``` {.sourceCode .js}
threelet.setup('mod-controls', THREE.OrbitControls); // enable controls
threelet.setup('mod-stats', window.Stats); // show the stats meter
threelet.setup('mod-webvr', window.WEBVR); // show the desktop/VR switch button
```

Sky based on the [shaders/sky
example](https://threejs.org/examples/?q=sky#webgl_shaders_sky) in
three.js:

``` {.sourceCode .html}
<script src="Sky.js"></script>

threelet.setup('mod-sky', THREE.Sky); // show sky with the analytical daylight
```

Build
-----

``` {.sourceCode .}
$ npm install  # set up build tools
$ npm run build  # generate module files in lib/
```
